pub struct EventLoop<T>
where T: 'static,
{ /* private fields */ }
Event Loop Provides a way to retrieve events from the system and from the windows that were registered to the events loop.

An EventLoop can be seen more or less as a “context”. Calling EventLoop::new initializes everything that will be required to create windows. For example on Linux creating an event loop opens a connection to the X or Wayland server.

To wake up an EventLoop from a another thread, see the EventLoopProxy docs.

Note that this cannot be shared across threads (due to platform-dependant logic forbidding it), as such it is neither Send nor Sync. If you need cross-thread access, the Window created from this can be sent to an other thread, and the EventLoopProxy allows you to wake up an EventLoop from another thread.

Implementations

source

impl EventLoop<()>

source

impl<T> EventLoop<T>

source

pub fn with_user_event() -> Result<EventLoop<T>, EventLoopError>

👎Deprecated: Use EventLoopBuilder::<T>::with_user_event().build() instead.
source

pub fn run<F>(self, event_handler: F) -> Result<(), EventLoopError>
where F: FnMut(Event<T>, &EventLoopWindowTarget<T>),

Runs the event loop in the calling thread and calls the given event_handler closure to dispatch any pending events.

Since the closure is 'static, it must be a move closure if it needs to access any data from the calling context.

See the set_control_flow() docs on how to change the event loop’s behavior.

Platform-specific
  • iOS: Will never return to the caller and so values not passed to this function will not be dropped before the process exits.

  • Web: Will act as if it never returns to the caller by throwing a Javascript exception (that Rust doesn’t see) that will also mean that the rest of the function is never executed and any values not passed to this function will not be dropped.

    Web applications are recommended to use EventLoopExtWebSys::spawn() 1 instead of run() to avoid the need for the Javascript exception trick, and to make it clearer that the event loop runs asynchronously (via the browser’s own, internal, event loop) and doesn’t block the current thread of execution like it does on other platforms.

    This function won’t be available with target_feature = "exception-handling".


  1. EventLoopExtWebSys::spawn() is only available on WASM. 

source

pub fn create_proxy(&self) -> EventLoopProxy<T>

Creates an EventLoopProxy that can be used to dispatch user events to the main event loop.

Methods from Deref<Target = EventLoopWindowTarget<T>>

source

pub fn available_monitors(&self) -> impl Iterator<Item = MonitorHandle>

Returns the list of all the monitors available on the system.

source

pub fn primary_monitor(&self) -> Option<MonitorHandle>

Returns the primary monitor of the system.

Returns None if it can’t identify any monitor as a primary one.

Platform-specific

Wayland / Web: Always returns None.

source

pub fn listen_device_events(&self, allowed: DeviceEvents)

Change if or when DeviceEvents are captured.

Since the DeviceEvent capture can lead to high CPU usage for unfocused windows, winit will ignore them by default for unfocused windows on Linux/BSD. This method allows changing this at runtime to explicitly capture them again.

Platform-specific
  • Wayland / macOS / iOS / Android / Orbital: Unsupported.
source

pub fn set_control_flow(&self, control_flow: ControlFlow)

Sets the ControlFlow.

source

pub fn control_flow(&self) -> ControlFlow

Gets the current ControlFlow.

source

pub fn exit(&self)

This exits the event loop.

See LoopExiting.

source

pub fn exiting(&self) -> bool

Returns if the EventLoop is about to stop.

See exit().