pub struct TextureView { /* private fields */ }
Surface Texture Handle to a texture view.

A TextureView object describes a texture and associated metadata needed by a RenderPipeline or BindGroup.

Corresponds to WebGPU GPUTextureView.

Implementations

source

impl TextureView

source

pub fn global_id(&self) -> Id<TextureView>

Returns a globally-unique identifier for this TextureView.

Calling this method multiple times on the same object will always return the same value. The returned value is guaranteed to be different for all resources created from the same Instance.