#[repr(C)]pub struct RequestAdapterOptionsBase<S> {
pub power_preference: PowerPreference,
pub force_fallback_adapter: bool,
pub compatible_surface: Option<S>,
}Expand description
Options for requesting adapter.
Corresponds to WebGPU GPURequestAdapterOptions.
Fields
power_preference: PowerPreferencePower preference for the adapter.
force_fallback_adapter: boolIndicates that only a fallback adapter can be returned. This is generally a “software” implementation on the system.
compatible_surface: Option<S>Surface that is required to be presentable with the requested adapter. This does not create the surface, only guarantees that the adapter can present to said surface. For WebGL, this is strictly required, as an adapter can not be created without a surface.