Dx12UseFrameLatencyWaitableObject

Enum Dx12UseFrameLatencyWaitableObject 

pub enum Dx12UseFrameLatencyWaitableObject {
    None,
    Wait,
    DontWait,
}
Whether and how to use a waitable handle obtained from GetFrameLatencyWaitableObject.

Variants



None

Do not obtain a waitable handle and do not wait for it. The swapchain will be created without the DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT flag.



Wait

Obtain a waitable handle and wait for it before acquiring the next swapchain image.



DontWait

Create the swapchain with the DXGI_SWAP_CHAIN_FLAG_FRAME_LATENCY_WAITABLE_OBJECT flag and obtain a waitable handle, but do not wait for it before acquiring the next swapchain image. This is useful if the application wants to wait for the waitable object itself.

Implementations

Source

impl Dx12UseFrameLatencyWaitableObject

pub fn from_env() -> Option<Dx12UseFrameLatencyWaitableObject>

Choose whether to use a frame latency waitable object from the environment variable WGPU_DX12_USE_FRAME_LATENCY_WAITABLE_OBJECT.

Valid values, case insensitive:

  • None
  • Wait
  • DontWait
pub fn with_env(self) -> Dx12UseFrameLatencyWaitableObject

Takes the given setting, modifies it based on the WGPU_DX12_USE_FRAME_LATENCY_WAITABLE_OBJECT environment variable, and returns the result.

See from_env for more information.