pub struct EngineSettings {Show 14 fields
pub width: u32,
pub height: u32,
pub title: &'static str,
pub decorations: bool,
pub resizable: bool,
pub control_flow: ControlFlow,
pub power_preference: PowerPreference,
pub backends: Backends,
pub features: Features,
pub present_mode: PresentMode,
pub limits: Limits,
pub alpha_mode: CompositeAlphaMode,
pub desired_maximum_frame_latency: u32,
pub memory_hints: MemoryHints,
}Expand description
Descriptor and settings for a window.
Fields
width: u32The width of the window
height: u32The height of the window
title: &'static strThe title of the window
decorations: boolShould the window contain the keys like minimize, maximize, or resize?
resizable: boolShould the window be resizable
control_flow: ControlFlowControls how the events are processed
read more at winit::event_loop::ControlFlow
power_preference: PowerPreferenceDefine how much power should the app ask for
backends: BackendsThe backend to use for the draw
features: FeaturesThe features to be enabled on a backend
read more at wgpu::Features
present_mode: PresentModeThe presentation mode of renderer for things like VSync
read more at wgpu::PresentMode
limits: LimitsLimits to be required based on the generation of the GPU and the API.
read more at wgpu::Limits
alpha_mode: CompositeAlphaModeThe alpha mode which specifies how the alpha channel of the textures should be handled during compositing.
desired_maximum_frame_latency: u32The desired frame latency.
read more at wgpu::SurfaceConfiguration::desired_maximum_frame_latency
memory_hints: MemoryHintsHow the memory should be utilized
read more at wgpu::MemoryHints